@@ -7,6 +7,7 @@ from fugue.collections.partition import (
     PartitionCursor,
     PartitionSpec,
 )
+from fugue.constants import FUGUE_DEFAULT_CONF
 from fugue.dataframe import DataFrame, DataFrames
 from fugue.dataframe.array_dataframe import ArrayDataFrame
 from fugue.dataframe.dataframe import LocalDataFrame
@@ -76,7 +77,8 @@ class ExecutionEngine(ABC):
     """
 
     def __init__(self, conf: Any):
-        self._conf = ParamDict(conf)
+        _conf = ParamDict(conf)
+        self._conf = ParamDict({**FUGUE_DEFAULT_CONF, **_conf})
 
     @property
     def conf(self) -> ParamDict:
@@ -197,13 +199,28 @@ class ExecutionEngine(ABC):
 
     @abstractmethod
     def persist(
-        self, df: DataFrame, level: Any = None
+        self,
+        df: DataFrame,
+        lazy: bool = False,
+        **kwargs: Any,
     ) -> DataFrame:  # pragma: no cover
         """Force materializing and caching the dataframe
 
         :param df: the input dataframe
-        :param level: parameter to pass to the underlying persist implementation
+        :param lazy: ``True``: first usage of the output will trigger persisting
+          to happen; ``False`` (eager): persist is forced to happend immediately.
+          Default to ``False``
+        :param *args: parameter to pass to the underlying persist implementation
+        :param *kwargs: parameter to pass to the underlying persist implementation
         :return: the persisted dataframe
+
+        :Notice:
+
+        ``persist`` can only guarantee the persisted dataframe will be computed
+        for only once. However this doesn't mean the backend really breaks up the
+        execution dependency at the persisting point. Commonly, it doesn't cause
+        any issue, but if your execution graph is long, it may cause expected
+        problems for example, stack overflow.
         """
         raise NotImplementedError
 
@@ -326,6 +343,31 @@ class ExecutionEngine(ABC):
         """
         pass
 
+    @abstractmethod
+    def dropna(
+        self,
+        df: DataFrame,
+        metadata: Any = None,
+        how: str = "any",
+        thresh: int = None,
+        subset: List[str] = None,
+    ) -> DataFrame:  # pragma: no cover
+        """Drop NA recods from dataframe
+
+        :param df: DataFrame
+        :param metadata: dict-like object to add to the result dataframe,
+            defaults to None
+        :type metadata: Any, optional
+        :param how: 'any' or 'all'. 'any' drops rows that contain any nulls.
+          'all' drops rows that contain all nulls.
+        :param thresh: int, drops rows that have less than thresh non-null values
+        :param subset: list of columns to operate on
+
+        :return: DataFrame with NA records dropped
+        :rtype: DataFrame
+        """
+        pass
+
     def zip(
         self,
         df1: DataFrame,
