@@ -1,14 +1,21 @@
 # flake8: noqa
 from fugue.extensions._builtins.creators import CreateData, Load
-from fugue.extensions._builtins.outputters import AssertEqual, Save, Show
+from fugue.extensions._builtins.outputters import (
+    AssertEqual,
+    AssertNotEqual,
+    Save,
+    Show,
+)
 from fugue.extensions._builtins.processors import (
-    DropColumns,
-    Rename,
     Distinct,
+    DropColumns,
+    Dropna,
+    Rename,
     RunJoin,
     RunSetOperation,
     RunSQLSelect,
     RunTransformer,
+    SaveAndUse,
     SelectColumns,
     Zip,
 )
