@@ -4,26 +4,34 @@ from typing import Any, Dict, Iterable, List, Optional, Set, TypeVar
 
 from adagio.specs import WorkflowSpec
 from fugue.collections.partition import PartitionSpec
+from fugue.constants import (
+    FUGUE_CONF_WORKFLOW_AUTO_PERSIST,
+    FUGUE_CONF_WORKFLOW_AUTO_PERSIST_VALUE,
+)
 from fugue.dataframe import DataFrame
 from fugue.dataframe.dataframes import DataFrames
 from fugue.exceptions import FugueWorkflowError
 from fugue.extensions._builtins import (
     AssertEqual,
+    AssertNotEqual,
     CreateData,
+    Distinct,
     DropColumns,
+    Dropna,
     Load,
     Rename,
     RunJoin,
-    Distinct,
     RunSetOperation,
     RunSQLSelect,
     RunTransformer,
     Save,
+    SaveAndUse,
     SelectColumns,
     Show,
     Zip,
 )
 from fugue.extensions.transformer.convert import _to_transformer
+from fugue.workflow._checkpoint import FileCheckpoint, WeakCheckpoint
 from fugue.workflow._tasks import Create, FugueTask, Output, Process
 from fugue.workflow._workflow_context import (
     FugueWorkflowContext,
@@ -128,7 +136,7 @@ class WorkflowDataFrame(DataFrame):
 
         Please read the :ref:`Processor Tutorial <tutorial:/tutorials/processor.ipynb>`
 
-        :param using: processor-like object
+        :param using: processor-like object, can't be a string expression
         :param schema: |SchemaLikeObject|, defaults to None. The processor
           will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.output_schema`
@@ -141,6 +149,9 @@ class WorkflowDataFrame(DataFrame):
         :return: result dataframe
         :rtype: :class:`~.WorkflowDataFrame`
         """
+        assert_or_throw(
+            not isinstance(using, str), f"processor {using} can't be string expression"
+        )
         if pre_partition is None:
             pre_partition = self._metadata.get("pre_partition", PartitionSpec())
         df = self.workflow.process(
@@ -154,7 +165,7 @@ class WorkflowDataFrame(DataFrame):
 
         Please read the :ref:`Outputter Tutorial <tutorial:/tutorials/outputter.ipynb>`
 
-        :param using: outputter-like object
+        :param using: outputter-like object, can't be a string expression
         :param params: |ParamsLikeObject| to run the outputter, defaults to None.
           The outputter will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.params`
@@ -162,6 +173,9 @@ class WorkflowDataFrame(DataFrame):
           The outputter will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.partition_spec`
         """
+        assert_or_throw(
+            not isinstance(using, str), f"outputter {using} can't be string expression"
+        )
         if pre_partition is None:
             pre_partition = self._metadata.get("pre_partition", PartitionSpec())
         self.workflow.output(
@@ -214,6 +228,24 @@ class WorkflowDataFrame(DataFrame):
         """
         self.workflow.assert_eq(self, *dfs, **params)
 
+    def assert_not_eq(self, *dfs: Any, **params: Any) -> None:
+        """Wrapper of :meth:`fugue.workflow.workflow.FugueWorkflow.assert_not_eq` to
+        compare this dataframe with other dataframes.
+
+        :param dfs: |DataFramesLikeObject|
+        :param digits: precision on float number comparison, defaults to 8
+        :param check_order: if to compare the row orders, defaults to False
+        :param check_schema: if compare schemas, defaults to True
+        :param check_content: if to compare the row values, defaults to True
+        :param check_metadata: if to compare the dataframe metadatas, defaults to True
+        :param no_pandas: if true, it will compare the string representations of the
+          dataframes, otherwise, it will convert both to pandas dataframe to compare,
+          defaults to False
+
+        :raises AssertionError: if any dataframe is equal to the first dataframe
+        """
+        self.workflow.assert_not_eq(self, *dfs, **params)
+
     def transform(
         self: TDF,
         using: Any,
@@ -228,7 +260,7 @@ class WorkflowDataFrame(DataFrame):
         Please read the
         :ref:`Transformer Tutorial <tutorial:/tutorials/transformer.ipynb>`
 
-        :param using: transformer-like object
+        :param using: transformer-like object, can't be a string expression
         :param schema: |SchemaLikeObject|, defaults to None. The transformer
           will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.output_schema`
@@ -243,6 +275,10 @@ class WorkflowDataFrame(DataFrame):
         :return: the transformed dataframe
         :rtype: :class:`~.WorkflowDataFrame`
         """
+        assert_or_throw(
+            not isinstance(using, str),
+            f"transformer {using} can't be string expression",
+        )
         if pre_partition is None:
             pre_partition = self._metadata.get("pre_partition", PartitionSpec())
         df = self.workflow.transform(
@@ -438,24 +474,124 @@ class WorkflowDataFrame(DataFrame):
         df = self.workflow.process(self, using=Distinct)
         return self._to_self_type(df)
 
-    def checkpoint(self: TDF, namespace: Any = None) -> TDF:
-        """[CURRENTLY NO EFFECT] set checkpoint for the current dataframe
+    def dropna(self: TDF, **kwargs: Any) -> TDF:
+        """Drops records containing NA records
 
-        :return: dataframe loaded from checkpoint
-        :rtype: :class:`~.WorkflowDataFrame`
+        :param kwargs: parameters to pass to dropna (how, thresh, subset)
+        :return: dataframe with incomplete records dropped
         """
-        self._task.checkpoint(namespace)
+        df = self.workflow.process(self, using=Dropna, params=kwargs)
+        return self._to_self_type(df)
+
+    def weak_checkpoint(self: TDF, lazy: bool = False, **kwargs: Any) -> TDF:
+        """Cache the dataframe in memory
+
+        :param lazy: whether it is a lazy checkpoint, defaults to False (eager)
+        :param kwargs: paramteters for the underlying execution engine function
+        :return: the cached dataframe
+
+        :Notice:
+
+        Weak checkpoint in most cases is the best choice for caching a dataframe to
+        avoid duplicated computation. However it does not guarantee to break up the
+        the compute dependency for this dataframe, so when you have very complicated
+        compute, you may encounter issues such as stack overflow. Also, weak checkpoint
+        normally caches the dataframe in memory, if memory is a concern, then you should
+        consider :meth:`~.strong_checkpoint`
+        """
+        self._task.set_checkpoint(WeakCheckpoint(lazy=lazy, **kwargs))
         return self
 
-    def persist(self: TDF, level: Any = None) -> TDF:
+    def strong_checkpoint(
+        self: TDF,
+        lazy: bool = False,
+        partition: Any = None,
+        single: bool = False,
+        **kwargs: Any,
+    ) -> TDF:
+        """Cache the dataframe as a temporary file
+
+        :param lazy: whether it is a lazy checkpoint, defaults to False (eager)
+        :param partition: |PartitionLikeObject|, defaults to None.
+        :param single: force the output as a single file, defaults to False
+        :param kwargs: paramteters for the underlying execution engine function
+        :return: the cached dataframe
+
+        :Notice:
+
+        Strong checkpoint guarantees the output dataframe compute dependency is
+        from the temporary file. Use strong checkpoint only when
+        :meth:`~.weak_checkpoint` can't be used.
+
+        Strong checkpoint file will be removed after the execution of the workflow.
+        """
+        self._task.set_checkpoint(
+            FileCheckpoint(
+                deterministic=False,
+                lazy=lazy,
+                partition=partition,
+                single=single,
+                **kwargs,
+            )
+        )
+        return self
+
+    def deterministic_checkpoint(
+        self: TDF,
+        lazy: bool = False,
+        partition: Any = None,
+        single: bool = False,
+        namespace: Any = None,
+        **kwargs: Any,
+    ) -> TDF:
+        """Cache the dataframe as a temporary file
+
+        :param lazy: whether it is a lazy checkpoint, defaults to False (eager)
+        :param partition: |PartitionLikeObject|, defaults to None.
+        :param single: force the output as a single file, defaults to False
+        :param kwargs: paramteters for the underlying execution engine function
+        :param namespace: a value to control determinism, defaults to None.
+        :return: the cached dataframe
+
+        :Notice:
+
+        The difference vs :meth:`~.strong_checkpoint` is that this checkpoint is not
+        removed after execution, so it can take effect cross execution if the dependent
+        compute logic is not changed.
+        """
+        self._task.set_checkpoint(
+            FileCheckpoint(
+                deterministic=True,
+                lazy=lazy,
+                partition=partition,
+                single=single,
+                namespace=namespace,
+                **kwargs,
+            )
+        )
+        return self
+
+    def persist(self: TDF) -> TDF:
         """Persist the current dataframe
 
-        :param level: the parameter passed to the underlying framework, defaults to None
         :return: the persisted dataframe
         :rtype: :class:`~.WorkflowDataFrame`
+
+        :Notice:
+
+        ``persist`` can only guarantee the persisted dataframe will be computed
+        for only once. However this doesn't mean the backend really breaks up the
+        execution dependency at the persisting point. Commonly, it doesn't cause
+        any issue, but if your execution graph is long, it may cause expected
+        problems for example, stack overflow.
+
+        ``persist`` method is considered as weak checkpoint. Sometimes, it may be
+        necessary to use strong checkpint, which is :meth:`~.checkpoint`
         """
-        self._task.persist("" if level is None else level)
-        return self
+        return self.weak_checkpoint(lazy=False)
+
+    def checkpoint(self: TDF) -> TDF:
+        return self.strong_checkpoint(lazy=False)
 
     def broadcast(self: TDF) -> TDF:
         """Broadcast the current dataframe
@@ -615,6 +751,41 @@ class WorkflowDataFrame(DataFrame):
             params=dict(path=path, fmt=fmt, mode=mode, single=single, params=kwargs),
         )
 
+    def save_and_use(
+        self: TDF,
+        path: str,
+        fmt: str = "",
+        mode: str = "overwrite",
+        partition: Any = None,
+        single: bool = False,
+        **kwargs: Any,
+    ) -> TDF:
+        """Save this dataframe to a persistent storage and load back to use
+        in the following steps
+
+        :param path: output path
+        :param fmt: format hint can accept ``parquet``, ``csv``, ``json``,
+          defaults to None, meaning to infer
+        :param mode: can accept ``overwrite``, ``append``, ``error``,
+          defaults to "overwrite"
+        :param partition: |PartitionLikeObject|, how to partition the
+          dataframe before saving, defaults to empty
+        :param single: force the output as a single file, defaults to False
+        :param kwargs: parameters to pass to the underlying framework
+
+        For more details and examples, read
+        :ref:`Save & Load <tutorial:/tutorials/dag.ipynb#save-&-load>`.
+        """
+        if partition is None:
+            partition = self._metadata.get("pre_partition", PartitionSpec())
+        df = self.workflow.process(
+            self,
+            using=SaveAndUse,
+            pre_partition=partition,
+            params=dict(path=path, fmt=fmt, mode=mode, single=single, params=kwargs),
+        )
+        return self._to_self_type(df)
+
     @property
     def schema(self) -> Schema:  # pragma: no cover
         """
@@ -779,7 +950,7 @@ class FugueWorkflow(object):
 
         Please read the :ref:`Creator Tutorial <tutorial:/tutorials/creator.ipynb>`
 
-        :param using: creator-like object
+        :param using: creator-like object, can't be a string expression
         :param schema: |SchemaLikeObject|, defaults to None. The creator
           will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.output_schema`
@@ -791,6 +962,10 @@ class FugueWorkflow(object):
           :meth:`~fugue.extensions.context.ExtensionContext.partition_spec`
         :return: result dataframe
         """
+        assert_or_throw(
+            not isinstance(using, str),
+            f"creator {using} can't be string expression",
+        )
         task = Create(creator=using, schema=schema, params=params)
         return self.add(task)
 
@@ -807,7 +982,7 @@ class FugueWorkflow(object):
         Please read the :ref:`Processor Tutorial <tutorial:/tutorials/processor.ipynb>`
 
         :param dfs: |DataFramesLikeObject|
-        :param using: processor-like object
+        :param using: processor-like object, can't be a string expression
         :param schema: |SchemaLikeObject|, defaults to None. The processor
           will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.output_schema`
@@ -820,6 +995,10 @@ class FugueWorkflow(object):
 
         :return: result dataframe
         """
+        assert_or_throw(
+            not isinstance(using, str),
+            f"processor {using} can't be string expression",
+        )
         dfs = self._to_dfs(*dfs)
         task = Process(
             len(dfs),
@@ -841,7 +1020,7 @@ class FugueWorkflow(object):
 
         Please read the :ref:`Outputter Tutorial <tutorial:/tutorials/outputter.ipynb>`
 
-        :param using: outputter-like object
+        :param using: outputter-like object, can't be a string expression
         :param params: |ParamsLikeObject| to run the outputter, defaults to None.
           The outputter will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.params`
@@ -849,6 +1028,10 @@ class FugueWorkflow(object):
           The outputter will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.partition_spec`
         """
+        assert_or_throw(
+            not isinstance(using, str),
+            f"outputter {using} can't be string expression",
+        )
         dfs = self._to_dfs(*dfs)
         task = Output(
             len(dfs),
@@ -1076,7 +1259,7 @@ class FugueWorkflow(object):
         :ref:`Transformer Tutorial <tutorial:/tutorials/transformer.ipynb>`
 
         :param dfs: |DataFramesLikeObject|
-        :param using: transformer-like object
+        :param using: transformer-like object, can't be a string expression
         :param schema: |SchemaLikeObject|, defaults to None. The transformer
           will be able to access this value from
           :meth:`~fugue.extensions.context.ExtensionContext.output_schema`
@@ -1090,6 +1273,10 @@ class FugueWorkflow(object):
           defaults to empty list
         :return: the transformed dataframe
         """
+        assert_or_throw(
+            not isinstance(using, str),
+            f"transformer {using} can't be string expression",
+        )
         assert_or_throw(
             len(dfs) == 1,
             NotImplementedError("transform supports only single dataframe"),
@@ -1142,7 +1329,7 @@ class FugueWorkflow(object):
         )
 
     def assert_eq(self, *dfs: Any, **params: Any) -> None:
-        """Compare if these dataframes are equal. Is for internal, unit test
+        """Compare if these dataframes are equal. It's for internal, unit test
         purpose only. It will convert both dataframes to
         :class:`~fugue.dataframe.dataframe.LocalBoundedDataFrame`, so it assumes
         all dataframes are small and fast enough to convert. DO NOT use it
@@ -1162,6 +1349,27 @@ class FugueWorkflow(object):
         """
         self.output(*dfs, using=AssertEqual, params=params)
 
+    def assert_not_eq(self, *dfs: Any, **params: Any) -> None:
+        """Assert if all dataframes are not equal to the first dataframe.
+        It's for internal, unit test purpose only. It will convert both dataframes to
+        :class:`~fugue.dataframe.dataframe.LocalBoundedDataFrame`, so it assumes
+        all dataframes are small and fast enough to convert. DO NOT use it
+        on critical or expensive tasks.
+
+        :param dfs: |DataFramesLikeObject|
+        :param digits: precision on float number comparison, defaults to 8
+        :param check_order: if to compare the row orders, defaults to False
+        :param check_schema: if compare schemas, defaults to True
+        :param check_content: if to compare the row values, defaults to True
+        :param check_metadata: if to compare the dataframe metadatas, defaults to True
+        :param no_pandas: if true, it will compare the string representations of the
+          dataframes, otherwise, it will convert both to pandas dataframe to compare,
+          defaults to False
+
+        :raises AssertionError: if any dataframe equals to the first dataframe
+        """
+        self.output(*dfs, using=AssertNotEqual, params=params)
+
     def add(self, task: FugueTask, *args: Any, **kwargs: Any) -> WorkflowDataFrame:
         """This method should not be called directly by users. Use
         :meth:`~.create`, :meth:`~.process`, :meth:`~.output` instead
@@ -1174,11 +1382,16 @@ class FugueWorkflow(object):
         for v in dep.dependency.values():
             v = v.split(".")[0]
             self._graph.add(name, v)
-            if len(self._graph.down[v]) > 1 and self.conf.get(
-                "fugue.workflow.auto_persist", False
+            if len(self._graph.down[v]) > 1 and self.conf.get_or_throw(
+                FUGUE_CONF_WORKFLOW_AUTO_PERSIST, bool
             ):
-                self._spec.tasks[v]._persist = self.conf.get(
-                    "fugue.workflow.auto_persist_value", ""
+                self._spec.tasks[v].set_checkpoint(
+                    WeakCheckpoint(
+                        lazy=False,
+                        level=self.conf.get_or_none(
+                            FUGUE_CONF_WORKFLOW_AUTO_PERSIST_VALUE, object
+                        ),
+                    )
                 )
         return WorkflowDataFrame(self, wt)
 
