@@ -19,15 +19,6 @@ from fugue.execution.execution_engine import (
     ExecutionEngine,
     SQLEngine,
 )
-from fugue_spark._constants import FUGUE_SPARK_DEFAULT_CONF
-from fugue_spark.dataframe import SparkDataFrame
-from fugue_spark._utils.convert import to_schema, to_spark_schema, to_type_safe_input
-from fugue_spark._utils.io import SparkIO
-from fugue_spark._utils.partition import (
-    even_repartition,
-    hash_repartition,
-    rand_repartition,
-)
 from pyspark import StorageLevel
 from pyspark.rdd import RDD
 from pyspark.sql import SparkSession
@@ -39,6 +30,19 @@ from triad.utils.hash import to_uuid
 from triad.utils.iter import EmptyAwareIterable
 from triad.utils.threading import RunOnce
 
+from fugue_spark._constants import (
+    FUGUE_SPARK_CONF_USE_PANDAS_UDF,
+    FUGUE_SPARK_DEFAULT_CONF,
+)
+from fugue_spark._utils.convert import to_schema, to_spark_schema, to_type_safe_input
+from fugue_spark._utils.io import SparkIO
+from fugue_spark._utils.partition import (
+    even_repartition,
+    hash_repartition,
+    rand_repartition,
+)
+from fugue_spark.dataframe import SparkDataFrame
+
 _TO_SPARK_JOIN_MAP: Dict[str, str] = {
     "inner": "inner",
     "leftouter": "left_outer",
@@ -234,7 +238,7 @@ class SparkExecutionEngine(ExecutionEngine):
         on_init: Optional[Callable[[int, DataFrame], Any]] = None,
     ) -> DataFrame:
         if (
-            self.conf.get_or_throw("fugue.spark.use_pandas_udf", bool)
+            self.conf.get_or_throw(FUGUE_SPARK_CONF_USE_PANDAS_UDF, bool)
             and len(partition_spec.partition_by) > 0
             and not any(pa.types.is_nested(t) for t in Schema(output_schema).types)
         ):
@@ -254,8 +258,15 @@ class SparkExecutionEngine(ExecutionEngine):
     def broadcast(self, df: DataFrame) -> SparkDataFrame:
         return self._broadcast_func(self.to_df(df))
 
-    def persist(self, df: DataFrame, level: Any = None) -> SparkDataFrame:
-        return self._persist_func(self.to_df(df), level)
+    def persist(
+        self,
+        df: DataFrame,
+        lazy: bool = False,
+        **kwargs: Any,
+    ) -> SparkDataFrame:
+        return self._persist_func(
+            self.to_df(df), lazy=lazy, level=kwargs.get("level", None)
+        )
 
     def register(self, df: DataFrame, name: str) -> SparkDataFrame:
         return self._register_func(self.to_df(df), name)
@@ -345,6 +356,17 @@ class SparkExecutionEngine(ExecutionEngine):
         d = self.to_df(df).native.distinct()
         return self.to_df(d, df.schema, metadata)
 
+    def dropna(
+        self,
+        df: DataFrame,
+        metadata: Any = None,
+        how: str = "any",
+        thresh: int = None,
+        subset: List[str] = None,
+    ) -> DataFrame:
+        d = self.to_df(df).native.dropna(how=how, thresh=thresh, subset=subset)
+        return self.to_df(d, df.schema, metadata)
+
     def load_df(
         self,
         path: Union[str, List[str]],
@@ -381,14 +403,15 @@ class SparkExecutionEngine(ExecutionEngine):
         sdf = broadcast(df.native)
         return SparkDataFrame(sdf, df.schema, df.metadata)
 
-    def _persist(self, df: SparkDataFrame, level: Any) -> SparkDataFrame:
+    def _persist(self, df: SparkDataFrame, lazy: bool, level: Any) -> SparkDataFrame:
         if level is None:
             level = StorageLevel.MEMORY_AND_DISK
         if isinstance(level, str) and level in StorageLevel.__dict__:
             level = StorageLevel.__dict__[level]
         if isinstance(level, StorageLevel):
             df.native.persist()
-            self.log.info(f"Persist dataframe with {level}, count {df.count()}")
+            if not lazy:
+                self.log.info(f"Persist dataframe with {level}, count {df.count()}")
             return df
         raise ValueError(f"{level} is not supported persist type")  # pragma: no cover
 
