@@ -22,8 +22,12 @@ from fugue_sql._parse import FugueSQL, _to_tokens
 from fugue_sql.exceptions import FugueSQLError, FugueSQLSyntaxError
 from triad.collections.schema import Schema
 from triad.utils.assertion import assert_or_throw
-from triad.utils.convert import to_bool
+from triad.utils.convert import get_caller_global_local_vars, to_bool
 from triad.utils.pyarrow import to_pa_datatype
+from fugue.extensions.creator.convert import _to_creator
+from fugue.extensions.processor.convert import _to_processor
+from fugue.extensions.outputter.convert import _to_outputter
+from fugue.extensions.transformer.convert import _to_transformer
 
 
 class FugueSQLHooks(object):
@@ -203,18 +207,36 @@ class _VisitorBase(FugueSQLVisitor):
     def visitFuguePath(self, ctx: fp.FuguePathContext) -> Any:
         return eval(self.ctxToStr(ctx))
 
-    def visitFuguePersist(self, ctx: fp.FuguePersistContext) -> Any:
-        if ctx.checkpoint:
-            if ctx.ns:
-                return True, self.visit(ctx.ns)
-            return True, None
+    def visitFugueCheckpointWeak(self, ctx: fp.FugueCheckpointWeakContext) -> Any:
+        lazy = ctx.LAZY() is not None
+        data = self.get_dict(ctx, "params")
+        if "params" not in data:
+            return lambda x: x.persist()
         else:
-            if ctx.value is None:
-                return False, None
-            return False, self.visit(ctx.value)
+            return lambda x: x.weak_checkpoint(lazy=lazy, **data["params"])
 
-    def visitFuguePersistValue(self, ctx: fp.FuguePersistValueContext) -> Any:
-        return self.ctxToStr(ctx, delimit="")
+    def visitFugueCheckpointStrong(self, ctx: fp.FugueCheckpointStrongContext) -> Any:
+        lazy = ctx.LAZY() is not None
+        data = self.get_dict(ctx, "partition", "single", "params")
+        return lambda x: x.strong_checkpoint(
+            lazy=lazy,
+            partition=data.get("partition"),
+            single="single" in data,
+            **data.get("params", {}),
+        )
+
+    def visitFugueCheckpointDeterministic(
+        self, ctx: fp.FugueCheckpointDeterministicContext
+    ) -> Any:
+        lazy = ctx.LAZY() is not None
+        data = self.get_dict(ctx, "ns", "partition", "single", "params")
+        return lambda x: x.deterministic_checkpoint(
+            lazy=lazy,
+            partition=data.get("partition"),
+            single="single" in data,
+            namespace=data.get("ns"),
+            **data.get("params", {}),
+        )
 
     def visitFugueCheckpointNamespace(self, ctx: fp.FugueCheckpointNamespaceContext):
         return str(eval(self.ctxToStr(ctx)))
@@ -228,6 +250,8 @@ class _Extensions(_VisitorBase):
         workflow: FugueWorkflow,
         variables: Optional[Dict[str, WorkflowDataFrame]] = None,
         last: Optional[WorkflowDataFrame] = None,
+        global_vars: Optional[Dict[str, Any]] = None,
+        local_vars: Optional[Dict[str, Any]] = None,
     ):
         super().__init__(sql)
         self._workflow = workflow
@@ -236,6 +260,9 @@ class _Extensions(_VisitorBase):
             self._variables.update(variables)
         self._last: Optional[WorkflowDataFrame] = last
         self._hooks = hooks
+        self._global_vars, self._local_vars = get_caller_global_local_vars(
+            global_vars, local_vars
+        )
 
     @property
     def workflow(self) -> FugueWorkflow:
@@ -255,6 +282,14 @@ class _Extensions(_VisitorBase):
             raise FugueSQLError("latest dataframe does not exist")
         return self._last
 
+    @property
+    def global_vars(self) -> Dict[str, Any]:
+        return self._global_vars
+
+    @property
+    def local_vars(self) -> Dict[str, Any]:
+        return self._local_vars
+
     def visitFugueDataFrameSource(
         self, ctx: fp.FugueDataFrameSourceContext
     ) -> WorkflowDataFrame:
@@ -271,6 +306,8 @@ class _Extensions(_VisitorBase):
             workflow=self.workflow,
             variables=self.variables,
             last=self._last,
+            global_vars=self.global_vars,
+            local_vars=self.local_vars,
         )
         sub.visit(ctx.task)
         return sub.last
@@ -297,11 +334,16 @@ class _Extensions(_VisitorBase):
         if "dfs" not in data:
             data["dfs"] = DataFrames(self.last)
         p = data["params"]
+        using = _to_transformer(
+            p["using"],
+            schema=p.get("schema"),
+            global_vars=self.global_vars,
+            local_vars=self.local_vars,
+        )
         # ignore errors is not implemented
         return self.workflow.transform(
             data["dfs"],
-            using=p["using"],
-            schema=p.get("schema"),
+            using=using,
             params=p.get("params"),
             pre_partition=data.get("partition"),
         )
@@ -313,10 +355,15 @@ class _Extensions(_VisitorBase):
         if "dfs" not in data:
             data["dfs"] = DataFrames(self.last)
         p = data["params"]
+        using = _to_processor(
+            p["using"],
+            schema=p.get("schema"),
+            global_vars=self.global_vars,
+            local_vars=self.local_vars,
+        )
         return self.workflow.process(
             data["dfs"],
-            using=p["using"],
-            schema=p.get("schema"),
+            using=using,
             params=p.get("params"),
             pre_partition=data.get("partition"),
         )
@@ -324,9 +371,13 @@ class _Extensions(_VisitorBase):
     def visitFugueCreateTask(self, ctx: fp.FugueCreateTaskContext) -> WorkflowDataFrame:
         data = self.get_dict(ctx, "params")
         p = data["params"]
-        return self.workflow.create(
-            using=p["using"], schema=p.get("schema"), params=p.get("params")
+        using = _to_creator(
+            p["using"],
+            schema=p.get("schema"),
+            global_vars=self.global_vars,
+            local_vars=self.local_vars,
         )
+        return self.workflow.create(using=using, params=p.get("params"))
 
     def visitFugueCreateDataTask(
         self, ctx: fp.FugueCreateDataTaskContext
@@ -346,9 +397,14 @@ class _Extensions(_VisitorBase):
         data = self.get_dict(ctx, "dfs", "using", "params", "partition")
         if "dfs" not in data:
             data["dfs"] = DataFrames(self.last)
+        using = _to_outputter(
+            data["using"],
+            global_vars=self.global_vars,
+            local_vars=self.local_vars,
+        )
         self.workflow.output(
             data["dfs"],
-            using=data["using"],
+            using=using,
             params=data.get("params"),
             pre_partition=data.get("partition"),
         )
@@ -383,6 +439,46 @@ class _Extensions(_VisitorBase):
             **data.get("params", {}),
         )
 
+    def visitFugueSaveAndUseTask(self, ctx: fp.FugueSaveAndUseTaskContext):
+        data = self.get_dict(
+            ctx, "partition", "df", "m", "single", "fmt", "path", "params"
+        )
+        if "df" in data:
+            df = data["df"]
+        else:
+            df = self.last
+        return df.save_and_use(
+            path=data["path"],
+            fmt=data.get("fmt", ""),
+            mode=data["m"],
+            partition=data.get("partition"),
+            single="single" in data,
+            **data.get("params", {}),
+        )
+
+    def visitFugueDropColumnsTask(self, ctx: fp.FugueDropColumnsTaskContext):
+        data = self.get_dict(ctx, "cols", "df")
+        if "df" in data:
+            df = data["df"]
+        else:
+            df = self.last
+        return df.drop(
+            columns=data["cols"],
+            if_exists=ctx.IF() is not None,
+        )
+
+    def visitFugueDropnaTask(self, ctx: fp.FugueDropnaTaskContext):
+        data = self.get_dict(ctx, "cols", "df")
+        if "df" in data:
+            df = data["df"]
+        else:
+            df = self.last
+        params: Dict[str, Any] = {}
+        params["how"] = "any" if ctx.ANY() is not None else "all"
+        if "cols" in data:
+            params["subset"] = data["cols"]
+        return df.dropna(**params)
+
     def visitFugueLoadTask(self, ctx: fp.FugueLoadTaskContext) -> WorkflowDataFrame:
         data = self.get_dict(ctx, "fmt", "path", "params", "columns")
         return self.workflow.load(
@@ -447,6 +543,8 @@ class _Extensions(_VisitorBase):
                 workflow=self.workflow,
                 variables=self.variables,
                 last=last,
+                global_vars=self.global_vars,
+                local_vars=self.local_vars,
             )
             yield sub.visit(n)
 
@@ -529,24 +627,13 @@ class _Extensions(_VisitorBase):
                 yield from self._get_query_elements(n)
 
     def _process_assignable(self, df: WorkflowDataFrame, ctx: Tree):
-        data = self.get_dict(ctx, "assign", "persist", "broadcast")
+        data = self.get_dict(ctx, "assign", "checkpoint", "broadcast")
         if "assign" in data:
-            varname, sign = data["assign"]
+            varname, _ = data["assign"]
         else:
-            varname, sign = None, None
-        need_checkpoint = sign == "??"
-        if "persist" in data:
-            is_checkpoint, value = data["persist"]
-            if need_checkpoint or is_checkpoint:
-                assert_or_throw(
-                    is_checkpoint,
-                    FugueSQLSyntaxError("can't persist when checkpoint is specified"),
-                )
-                df = df.checkpoint(value)
-            else:
-                df = df.persist(value)
-        elif need_checkpoint:
-            df = df.checkpoint()
+            varname = None
+        if "checkpoint" in data:
+            df = data["checkpoint"](df)
         if "broadcast" in data:
             df = df.broadcast()
         if varname is not None:
