@@ -4,12 +4,14 @@ import pickle
 from typing import Any, Dict, Iterable, List
 from unittest import TestCase
 
+import numpy as np
 import pandas as pd
 import pytest
 from fugue import FileSystem, Schema
 from fugue.dataframe import DataFrame, DataFrames, LocalDataFrame, PandasDataFrame
 from fugue.dataframe.array_dataframe import ArrayDataFrame
 from fugue.dataframe.utils import _df_eq as df_eq
+from fugue.exceptions import FugueWorkflowError
 from fugue.execution import ExecutionEngine
 from fugue.execution.native_execution_engine import SqliteEngine
 from fugue.extensions.outputter import Outputter
@@ -21,6 +23,9 @@ from fugue.extensions.transformer import (
     transformer,
 )
 from fugue.workflow.workflow import FugueWorkflow, _FugueInteractiveWorkflow
+from pytest import raises
+from triad.utils.convert import get_full_type_path
+from fugue.collections.partition import PartitionSpec
 
 
 class BuiltInTests(object):
@@ -64,10 +69,123 @@ class BuiltInTests(object):
                 dag.df([[0]], "a:int").persist().partition(num=2).show()
                 dag.df(dag.df([[0]], "a:int")).persist().broadcast().show()
 
-        def test_checkpoint_no_effect(self):
+        def test_checkpoint(self):
+            with raises(FugueWorkflowError):
+                with self.dag() as dag:
+                    dag.df([[0]], "a:int").checkpoint()
+
+            self.engine.conf["fugue.workflow.checkpoint.path"] = os.path.join(
+                self.tmpdir, "ck"
+            )
             with self.dag() as dag:
-                dag.df([[0]], "a:int").checkpoint().partition(num=2).show()
-                dag.df(dag.df([[0]], "a:int")).checkpoint("dummy").broadcast().show()
+                a = dag.df([[0]], "a:int").checkpoint()
+                dag.df([[0]], "a:int").assert_eq(a)
+
+        def test_deterministic_checkpoint(self):
+            self.engine.conf["fugue.workflow.checkpoint.path"] = os.path.join(
+                self.tmpdir, "ck"
+            )
+            temp_file = os.path.join(self.tmpdir, "t.parquet")
+
+            def mock_create(dummy: int = 1) -> pd.DataFrame:
+                return pd.DataFrame(np.random.rand(3, 2), columns=["a", "b"])
+
+            # base case without checkpoint, two runs should generate different result
+            with self.dag() as dag:
+                a = dag.create(mock_create)
+                a.save(temp_file)
+            with self.dag() as dag:
+                a = dag.create(mock_create)
+                b = dag.load(temp_file)
+                b.assert_not_eq(a)
+
+            # strong checkpoint is not cross execution, so result should be different
+            with self.dag() as dag:
+                a = dag.create(mock_create).strong_checkpoint()
+                a.save(temp_file)
+            with self.dag() as dag:
+                a = dag.create(mock_create).strong_checkpoint()
+                b = dag.load(temp_file)
+                b.assert_not_eq(a)
+
+            # with deterministic checkpoint, two runs should generate the same result
+            with self.dag() as dag:
+                dag.create(
+                    mock_create, params=dict(dummy=2)
+                )  # this should not affect a because it's not a dependency
+                a = dag.create(mock_create).deterministic_checkpoint()
+                a.save(temp_file)
+            with self.dag() as dag:
+                a = dag.create(mock_create).deterministic_checkpoint()
+                b = dag.load(temp_file)
+                b.assert_eq(a)
+            with self.dag() as dag:
+                # even change on the settings of deterministic checkpoint
+                # will affect determinism
+                a = dag.create(mock_create).deterministic_checkpoint(
+                    partition=PartitionSpec(num=2)
+                )
+                b = dag.load(temp_file)
+                b.assert_not_eq(a)
+
+        def test_deterministic_checkpoint_complex_dag(self):
+            self.engine.conf["fugue.workflow.checkpoint.path"] = os.path.join(
+                self.tmpdir, "ck"
+            )
+            temp_file = os.path.join(self.tmpdir, "t.parquet")
+
+            def mock_create(dummy: int = 1) -> pd.DataFrame:
+                return pd.DataFrame(np.random.rand(3, 2), columns=["a", "b"])
+
+            # base case without checkpoint, two runs should generate different result
+            with self.dag() as dag:
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b)
+                c.save(temp_file)
+            with self.dag() as dag:
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b)
+                d = dag.load(temp_file)
+                d.assert_not_eq(c)
+
+            # strong checkpoint is not cross execution, so result should be different
+            with self.dag() as dag:
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b).strong_checkpoint()
+                c.save(temp_file)
+            with self.dag() as dag:
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b).strong_checkpoint()
+                d = dag.load(temp_file)
+                d.assert_not_eq(c)
+
+            # with deterministic checkpoint, two runs should generate the same result
+            with self.dag() as dag:
+                dag.create(
+                    mock_create, params=dict(dummy=2)
+                )  # this should not affect a because it's not a dependency
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b).deterministic_checkpoint()
+                c.save(temp_file)
+            with self.dag() as dag:
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b).deterministic_checkpoint()
+                d = dag.load(temp_file)
+                d.assert_eq(c)
+            with self.dag() as dag:
+                # even change on the settings of deterministic checkpoint
+                # will affect determinism
+                a = dag.create(mock_create).drop(["a"])
+                b = dag.create(mock_create).drop(["a"])
+                c = a.union(b).deterministic_checkpoint(partition=PartitionSpec(num=2))
+                d = dag.load(temp_file)
+                d.assert_not_eq(c)
 
         def test_create_process_output(self):
             with self.dag() as dag:
@@ -346,7 +464,7 @@ class BuiltInTests(object):
                     "AS t1 INNER JOIN",
                     b,
                     "AS t2 ON t1.x=t2.x",
-                    sql_engine="SqliteEngine",
+                    sql_engine=get_full_type_path(SqliteEngine),
                 ).assert_eq(c)
 
                 # no input
@@ -464,6 +582,33 @@ class BuiltInTests(object):
                     )
                 )
 
+        def test_dropna(self):
+            with self.dag() as dag:
+                a = dag.df(
+                    [[1, 10, 10], [None, 2, None], [2, None, 4]],
+                    "x:double,y:double,z:double",
+                )
+                a.dropna().assert_eq(
+                    ArrayDataFrame(
+                        [[1, 10, 10]],
+                        "x:double,y:double,z:double",
+                    )
+                )
+                a.dropna(how="all").assert_eq(a)
+                a.dropna(thresh=2).assert_eq(
+                    ArrayDataFrame(
+                        [[1, 10, 10], [2, None, 4]],
+                        "x:double,y:double,z:double",
+                    )
+                )
+                a.dropna(how="any", subset=["x", "z"]).assert_eq(
+                    ArrayDataFrame(
+                        [[1, 10, 10], [2, None, 4]],
+                        "x:double,y:double,z:double",
+                    )
+                )
+                a.dropna(thresh=1, subset=["y", "z"]).assert_eq(a)
+
         def test_col_ops(self):
             with self.dag() as dag:
                 a = dag.df([[1, 10], [2, 20]], "x:long,y:long")
@@ -492,7 +637,7 @@ class BuiltInTests(object):
 
         def test_io(self):
             path = os.path.join(self.tmpdir, "a")
-            path2 = os.path.join(self.tmpdir, "b.csv")
+            path2 = os.path.join(self.tmpdir, "b.test.csv")
             with self.dag() as dag:
                 b = dag.df([[6, 1], [2, 7]], "c:int,a:long")
                 b.partition(num=3).save(path, fmt="parquet", single=True)
@@ -504,6 +649,18 @@ class BuiltInTests(object):
                 a = dag.load(path2, header=True, columns="c:int,a:long")
                 a.assert_eq(dag.df([[6, 1], [2, 7]], "c:int,a:long"))
 
+        def test_save_and_use(self):
+            path = os.path.join(self.tmpdir, "a")
+            with self.dag() as dag:
+                b = dag.df([[6, 1], [2, 7]], "c:int,a:long")
+                c = b.save_and_use(path, fmt="parquet")
+                b.assert_eq(c)
+
+            with self.dag() as dag:
+                b = dag.df([[6, 1], [2, 7]], "c:int,a:long")
+                d = dag.load(path, fmt="parquet")
+                b.assert_eq(d)
+
 
 def mock_creator(p: int) -> DataFrame:
     return ArrayDataFrame([[p]], "a:int")
